@@ -18,20 +18,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-package Dpkg::Vendor::Debian;
-
-use strict;
-use warnings;
-
-our $VERSION = '0.01';
-
-use Dpkg;
-use Dpkg::Gettext;
-use Dpkg::ErrorHandling;
-use Dpkg::Control::Types;
-
-use parent qw(Dpkg::Vendor::Default);
-
 =encoding utf8
 
 =head1 NAME
@@ -40,11 +26,27 @@ Dpkg::Vendor::Debian - Debian vendor class
 
 =head1 DESCRIPTION
 
-This vendor class customizes the behaviour of dpkg scripts for Debian
+This vendor class customizes the behavior of dpkg scripts for Debian
 specific behavior and policies.
 
+B<Note>: This is a private module, its API can change at any time.
+
 =cut
 
+package Dpkg::Vendor::Debian 0.01;
+
+use strict;
+use warnings;
+
+use List::Util qw(any none);
+
+use Dpkg;
+use Dpkg::Gettext;
+use Dpkg::ErrorHandling;
+use Dpkg::Control::Types;
+
+use parent qw(Dpkg::Vendor::Default);
+
 sub run_hook {
     my ($self, $hook, @params) = @_;
 
@@ -94,14 +96,25 @@ sub run_hook {
     }
 }
 
+sub init_build_features {
+    my ($self, $use_feature, $builtin_feature) = @_;
+}
+
 sub set_build_features {
     my ($self, $flags) = @_;
 
     # Default feature states.
     my %use_feature = (
         future => {
+            # XXX: Should start a deprecation cycle at some point.
             lfs => 0,
         },
+        abi => {
+            # XXX: This is set to undef so that we can handle the alias from
+            # the future feature area.
+            lfs => undef,
+            time64 => 0,
+        },
         qa => {
             bug => 0,
             canary => 0,
@@ -126,19 +139,121 @@ sub set_build_features {
             pie => undef,
             stackprotector => 1,
             stackprotectorstrong => 1,
+            stackclash => 1,
             fortify => 1,
             format => 1,
             relro => 1,
             bindnow => 0,
+            branch => 1,
         },
     );
 
     my %builtin_feature = (
+        abi => {
+            lfs => 0,
+            time64 => 0,
+        },
         hardening => {
             pie => 1,
         },
     );
 
+    require Dpkg::Arch;
+
+    my $arch = Dpkg::Arch::get_host_arch();
+    my ($abi, $libc, $os, $cpu) = Dpkg::Arch::debarch_to_debtuple($arch);
+    my ($abi_bits, $abi_endian) = Dpkg::Arch::debarch_to_abiattrs($arch);
+
+    unless (defined $abi and defined $libc and defined $os and defined $cpu) {
+        warning(g_("unknown host architecture '%s'"), $arch);
+        ($abi, $os, $cpu) = ('', '', '');
+    }
+    unless (defined $abi_bits and defined $abi_endian) {
+        warning(g_("unknown abi attributes for architecture '%s'"), $arch);
+        ($abi_bits, $abi_endian) = (0, 'unknown');
+    }
+
+    # Mask builtin features that are not enabled by default in the compiler.
+    my %builtin_pie_arch = map { $_ => 1 } qw(
+        amd64
+        arm64
+        armel
+        armhf
+        hurd-amd64
+        hurd-i386
+        i386
+        kfreebsd-amd64
+        kfreebsd-i386
+        loong64
+        mips
+        mips64
+        mips64el
+        mips64r6
+        mips64r6el
+        mipsel
+        mipsn32
+        mipsn32el
+        mipsn32r6
+        mipsn32r6el
+        mipsr6
+        mipsr6el
+        powerpc
+        ppc64
+        ppc64el
+        riscv64
+        s390x
+        sparc
+        sparc64
+    );
+    if (not exists $builtin_pie_arch{$arch}) {
+        $builtin_feature{hardening}{pie} = 0;
+    }
+
+    if ($abi_bits != 32) {
+        $builtin_feature{abi}{lfs} = 1;
+    }
+
+    # On glibc, new ports default to time64, old ports currently default
+    # to time32, so we track the latter as that is a list that is not
+    # going to grow further, and might shrink.
+    # On musl libc based systems all ports use time64.
+    my %time32_arch = map { $_ => 1 } qw(
+        arm
+        armeb
+        armel
+        armhf
+        hppa
+        i386
+        hurd-i386
+        kfreebsd-i386
+        m68k
+        mips
+        mipsel
+        mipsn32
+        mipsn32el
+        mipsn32r6
+        mipsn32r6el
+        mipsr6
+        mipsr6el
+        nios2
+        powerpc
+        powerpcel
+        powerpcspe
+        s390
+        sh3
+        sh3eb
+        sh4
+        sh4eb
+        sparc
+    );
+    if ($abi_bits != 32 or
+        not exists $time32_arch{$arch} or
+        $libc eq 'musl') {
+        $builtin_feature{abi}{time64} = 1;
+    }
+
+    $self->init_build_features(\%use_feature, \%builtin_feature);
+
     ## Setup
 
     require Dpkg::BuildOptions;
@@ -152,26 +267,18 @@ sub set_build_features {
         $opts_maint->parse_features($area, $use_feature{$area});
     }
 
-    require Dpkg::Arch;
+    ## Area: abi
 
-    my $arch = Dpkg::Arch::get_host_arch();
-    my ($abi, $libc, $os, $cpu) = Dpkg::Arch::debarch_to_debtuple($arch);
-
-    unless (defined $abi and defined $libc and defined $os and defined $cpu) {
-        warning(g_("unknown host architecture '%s'"), $arch);
-        ($abi, $os, $cpu) = ('', '', '');
+    if ($use_feature{abi}{time64} && ! $builtin_feature{abi}{time64}) {
+        # On glibc 64-bit time_t support requires LFS.
+        $use_feature{abi}{lfs} = 1 if $libc eq 'gnu';
     }
 
-    ## Area: future
-
-    if ($use_feature{future}{lfs}) {
-        my ($abi_bits, $abi_endian) = Dpkg::Arch::debarch_to_abiattrs($arch);
-        my $cpu_bits = Dpkg::Arch::debarch_to_cpubits($arch);
-
-        if ($abi_bits != 32 or $cpu_bits != 32) {
-            $use_feature{future}{lfs} = 0;
-        }
-    }
+    # XXX: Handle lfs alias from future abi feature area.
+    $use_feature{abi}{lfs} //= $use_feature{future}{lfs};
+    # XXX: Once the feature is set in the abi area, we always override the
+    # one in the future area.
+    $use_feature{future}{lfs} = $use_feature{abi}{lfs};
 
     ## Area: reproducible
 
@@ -216,50 +323,34 @@ sub set_build_features {
 
     ## Area: hardening
 
-    # Mask builtin features that are not enabled by default in the compiler.
-    my %builtin_pie_arch = map { $_ => 1 } qw(
-        amd64
-        arm64
-        armel
-        armhf
-        hurd-i386
-        i386
-        kfreebsd-amd64
-        kfreebsd-i386
-        mips
-        mipsel
-        mips64el
-        powerpc
-        ppc64
-        ppc64el
-        riscv64
-        s390x
-        sparc
-        sparc64
-    );
-    if (not exists $builtin_pie_arch{$arch}) {
-        $builtin_feature{hardening}{pie} = 0;
-    }
-
     # Mask features that are not available on certain architectures.
-    if ($os !~ /^(?:linux|kfreebsd|knetbsd|hurd)$/ or
-	$cpu =~ /^(?:hppa|avr32)$/) {
+    if (none { $os eq $_ } qw(linux kfreebsd knetbsd hurd) or
+        $cpu eq 'hppa') {
 	# Disabled on non-(linux/kfreebsd/knetbsd/hurd).
-	# Disabled on hppa, avr32
-	#  (#574716).
+        # Disabled on hppa.
 	$use_feature{hardening}{pie} = 0;
     }
-    if ($cpu =~ /^(?:ia64|alpha|hppa|nios2)$/ or $arch eq 'arm') {
+    if (any { $cpu eq $_ } qw(ia64 alpha hppa nios2) or $arch eq 'arm') {
 	# Stack protector disabled on ia64, alpha, hppa, nios2.
 	#   "warning: -fstack-protector not supported for this target"
 	# Stack protector disabled on arm (ok on armel).
 	#   compiler supports it incorrectly (leads to SEGV)
 	$use_feature{hardening}{stackprotector} = 0;
     }
-    if ($cpu =~ /^(?:ia64|hppa|avr32)$/) {
-	# relro not implemented on ia64, hppa, avr32.
+    if (none { $arch eq $_ } qw(amd64 arm64 armhf armel)) {
+        # Stack clash protector only available on amd64 and arm.
+        $use_feature{hardening}{stackclash} = 0;
+    }
+    if (any { $cpu eq $_ } qw(ia64 hppa)) {
+	# relro not implemented on ia64, hppa.
 	$use_feature{hardening}{relro} = 0;
     }
+    if (none { $cpu eq $_ } qw(amd64 arm64)) {
+        # On amd64 use -fcf-protection.
+        # On arm64 use -mbranch-protection=standard.
+        $use_feature{hardening}{branch} = 0;
+    }
+    $flags->set_option_value('hardening-branch-cpu', $cpu);
 
     # Mask features that might be influenced by other flags.
     if ($flags->get_option_value('optimize-level') == 0) {
@@ -307,7 +398,6 @@ sub _add_build_flags {
         OBJCXXFLAGS
         FFLAGS
         FCFLAGS
-        GCJFLAGS
     );
 
     my $default_flags;
@@ -322,15 +412,22 @@ sub _add_build_flags {
     }
 
     $flags->append($_, $default_flags) foreach @compile_flags;
+    $flags->append($_ . '_FOR_BUILD', $default_flags) foreach @compile_flags;
     $flags->append('DFLAGS', $default_d_flags);
+    $flags->append('DFLAGS_FOR_BUILD', $default_d_flags);
 
-    ## Area: future
+    ## Area: abi
 
-    if ($flags->use_feature('future', 'lfs')) {
+    my %abi_builtins = $flags->get_builtins('abi');
+    if ($flags->use_feature('abi', 'lfs') && ! $abi_builtins{lfs}) {
         $flags->append('CPPFLAGS',
                        '-D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64');
     }
 
+    if ($flags->use_feature('abi', 'time64') && ! $abi_builtins{time64}) {
+        $flags->append('CPPFLAGS', '-D_TIME_BITS=64');
+    }
+
     ## Area: qa
 
     # Warnings that detect actual bugs.
@@ -427,7 +524,7 @@ sub _add_build_flags {
     ## Area: hardening
 
     # PIE
-    my $use_pie = $flags->use_feature('hardening', 'pie');
+    my $use_pie = $flags->get_feature('hardening', 'pie');
     my %hardening_builtins = $flags->get_builtins('hardening');
     if (defined $use_pie && $use_pie && ! $hardening_builtins{pie}) {
 	my $flag = "-specs=$Dpkg::DATADIR/pie-compile.specs";
@@ -448,6 +545,12 @@ sub _add_build_flags {
         $flags->append($_, $flag) foreach @compile_flags;
     }
 
+    # Stack clash
+    if ($flags->use_feature('hardening', 'stackclash')) {
+        my $flag = '-fstack-clash-protection';
+        $flags->append($_, $flag) foreach @compile_flags;
+    }
+
     # Fortify Source
     if ($flags->use_feature('hardening', 'fortify')) {
 	$flags->append('CPPFLAGS', '-D_FORTIFY_SOURCE=2');
@@ -471,6 +574,18 @@ sub _add_build_flags {
     if ($flags->use_feature('hardening', 'bindnow')) {
 	$flags->append('LDFLAGS', '-Wl,-z,now');
     }
+
+    # Branch protection
+    if ($flags->use_feature('hardening', 'branch')) {
+        my $cpu = $flags->get_option_value('hardening-branch-cpu');
+        my $flag;
+        if ($cpu eq 'arm64') {
+            $flag = '-mbranch-protection=standard';
+        } elsif ($cpu eq 'amd64') {
+            $flag = '-fcf-protection';
+        }
+        $flags->append($_, $flag) foreach @compile_flags;
+    }
 }
 
 sub _build_tainted_by {
